/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3
Created on Date : 09/03/2012
Last update     : 04/10/2013

CopyRight 2006-2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller via an I2C interface made with a PCF8574							
 	
Before using the functions you have to properly initialize the I2C module to 100KHz.
hardware connetions are (using Eagle CAD nomenclature):

PCF8574 P0 -> LCD D4 
PCF8574 P1 -> LCD D5 
PCF8574 P2 -> LCD D6 
PCF8574 P3 -> LCD D7 
PCF8574 P4 -> LCD E 
PCF8574 P5 -> LCD R/W 
PCF8574 P6 -> LCD RS 
PCF8574 P7 -> LCD LED (use a PNP transistor as buffer to drive the backlight LED).

SDA, SCL must be connected to the I2C buffer with pull-up resistors.
A0-A1-A2 adderss pins must be set as required.
Set PCF8574_ADDRESS_L (see below)  accordengly to the address pis A0-A1-A2.

More details and pictures can be downloaded from the Brief Note BN0015 (www.LaurTec.com)

**********************************************************************************************/


#ifndef LCD_44780_I2C_H
#define LCD_44780_I2C_H

#ifdef __XC8
    #include <xc.h>
    #include <stdlib.h>

    #ifndef _PIC18
        #error "The LCD_44780_I2C Library supports only PIC18 devices"
    #endif
#endif

#include <delay.h>

#ifdef __18CXX
        #include <i2c.h>
#endif

#ifndef __XC8
    #include <ctype.h>
#endif


//**************************************************
// LCD constants
// Each bit is connected to the PCF8574 
//**************************************************


#define LCD_D0  0b00000001
#define LCD_D1  0b00000010
#define LCD_D2  0b00000100	
#define LCD_D3  0b00001000	
#define LCD_E   0b00010000	
#define LCD_RW  0b00100000
#define LCD_RS  0b01000000	
#define LCD_LED 0b10000000
	
//**************************************************
// PCF8574 Address
// PCF8574_ADDRESS_H refers to the enbedded address (PCF8574 model)
// PCF8574_ADDRESS_L refers to the A0, A1, A2 pins
//**************************************************	

//PCF8574
//#define PCF8574_ADDRESS_H 0x40

//PCF8574A
#define PCF8574_ADDRESS_H 0x70

#define PCF8574_ADDRESS_L 0x00 

//**************************************************
//               Constant Definitions

#define LEFT  0b00000000
#define RIGHT 0b00000100

#define TURN_ON_CURSOR  0b00000010
#define TURN_OFF_CURSOR 0b00000000

#define TURN_ON_LED_LCD  0b00000000
#define TURN_OFF_LED_LCD 0b10000000

#define BLINKING_ON  0b00000001
#define BLINKING_OFF 0b00000000

//**************************************************



/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void enable_pulse_LCD (void);
#define Epulse enable_pulse_LCD


/**
 * This function generates the Enable pulse.
 *
 *
 * @param data Data to be sent [0..255]
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void send_command_LCD (unsigned char data);
#define SendCommand send_command_LCD


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void home_LCD(void);
#define HomeLCD home_LCD


/**
 * This function shifts the LCD screen on the left or rigt.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780 library.
 */
void shift_LCD(unsigned char shift, unsigned char number_of_shift);
#define ShiftLCD shift_LCD


/**
 * This function shifts the LCD cursor on the left or rigt. The position of the cursor is where the
 * next writting will be performed.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780 library.
 */
void shift_cursor_LCD(unsigned char shift,unsigned char number_of_shift);
#define ShiftCursorLCD shift_cursor_LCD


/**
 * This function locates the LCD cursor on the selected line. Tested on 20x4 16x2 LCD displays.
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Dipslays.
 */
void goto_line_LCD (unsigned char line);
#define GotoLineLCD goto_line_LCD


/**
 * This function locates the LCD cursor to an arbitrary X Y location.
 * Y represents the line number from top.
 *
 * @param x It Specifies horizontal position
 *
 * @param y It Specifies vertical position (line number)
 *
 * @warning It might not work with all the LCD Dipslays. Tested on 20x4 16x2 LCD displays.
 */
void goto_xy_LCD (unsigned char x, unsigned char y);


/**
 * This function writes a char to the LCD display. (e.g WriteCharLCD ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void write_char_LCD (unsigned char value);
#define WriteCharLCD write_char_LCD



/**
 * This function writes a const string to the LCD display, e.g write_message_LCD("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write arrays refers the write_string_LCD () function.
 *
 */
#ifndef __XC8
void write_message_LCD(const rom unsigned char *buffer);
#endif

#ifdef __XC8
void write_message_LCD(const unsigned char *buffer);
#endif

#define WriteStringLCD write_message_LCD



/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within rom use write_message_LCD ().
 *
 */
void write_string_LCD(unsigned char *buffer);
#define WriteVarLCD write_string_LCD


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD diplay.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-5].
 *                    0: Left Justified
 *                    1-5: Right Justified with n digit
 *
 * @note If you set a number of digit less than required the digit will be lost starting from the less
 *       significant digit. Minus is like a digit.
 *
 */
void write_integer_LCD(int value, unsigned char number_of_digits);
#define WriteIntLCD write_integer_LCD


/**
 * This Function cleans the LCD display.
 *
 */
void clear_LCD (void);
#define ClearLCD clear_LCD


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [TURN_ON_CURSOR, TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [BLINK_ON,BLINK_OFF]
 *
 * @note You must use the constant TURN_ON_CURSOR, TURN_OFF_CURSOR, BLINKING_ON, BLINKING_OFF
 *       to ensure the compatibility with LCD_44780 library.
 *
 */
void cursor_LCD(unsigned char active, unsigned char blinking);
#define CursorLCD cursor_LCD


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [TURN_ON_LED_LCD, TURN_OFF_LED_LCD]
 *
 * @param blinking  Let the cursor blink [BLINKING_ON,BLINKING_OFF]
 *
 * @note You must use the constant TURN_ON_LED_LCD, TURN_OFF_LED_LCD
 *       to ensure the compatibility with LCD_44780 library.
 */
void backlight_LCD(unsigned char active);
#define BacklightLCD backlight_LCD


/**
 * This funnction initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  initialize_LCD(unsigned char quartz_frequency);
#define OpenLCD initialize_LCD


#endif
